package ink.wgink.redis.cache.manager.app;

import ink.wgink.interfaces.manager.IAppTokenService;
import ink.wgink.pojo.app.AppToken;
import ink.wgink.pojo.app.AppTokenUser;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Primary;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: RedisAppTokenManager
 * @Description: redis app token 管理器
 * @Author: wanggeng
 * @Date: 2021/11/29 4:25 下午
 * @Version: 1.0
 */
@Primary
@Service
public class RedisAppTokenService implements IAppTokenService {

    private static final Logger LOG = LoggerFactory.getLogger(RedisAppTokenService.class);
    @Autowired
    private RedisTemplate redisTemplate;

    @Override
    public AppToken getToken(String token) {
        AppToken appToken = (AppToken) redisTemplate.opsForHash().entries(TOKEN_HASH_KEY).get(token);
        if (appToken != null) {
            appToken.setLastTime(System.currentTimeMillis());
            redisTemplate.opsForHash().put(TOKEN_HASH_KEY, appToken.getToken(), appToken);
        }
        return appToken;
    }

    @Override
    public void addToken(String token, AppToken.AppTokenTypeEnum appTokenTypeEnum, AppTokenUser appTokenUser) {
        AppToken appToken = new AppToken();
        appToken.setToken(token);
        appToken.setAppTokenTypeEnum(appTokenTypeEnum);
        appToken.setLastTime(System.currentTimeMillis());
        appToken.setAppTokenUser(appTokenUser);
        appToken.setUserId(appTokenUser.getId());
        Map<String, AppToken> tokens = redisTemplate.opsForHash().entries(TOKEN_HASH_KEY);
        for (Map.Entry<String, AppToken> kvs : tokens.entrySet()) {
            if (StringUtils.equals(appToken.getAppTokenUser().getId(), kvs.getValue().getUserId())) {
                redisTemplate.opsForHash().delete(TOKEN_HASH_KEY, kvs.getValue().getToken());
                break;
            }
        }
        redisTemplate.opsForHash().put(TOKEN_HASH_KEY, appToken.getToken(), appToken);
    }

    @Override
    public List<AppTokenUser> listCurrentUsers() {
        List<AppTokenUser> users = new ArrayList<>();
        Map<String, AppToken> tokens = redisTemplate.opsForHash().entries(TOKEN_HASH_KEY);
        for (Map.Entry<String, AppToken> kvs : tokens.entrySet()) {
            AppToken appToken = kvs.getValue();
            users.add(appToken.getAppTokenUser());
        }
        return users;
    }

    @Override
    public void clearTimeoutToken() {
        long currentTime = System.currentTimeMillis();
        List<String> clearTokenKeys = new ArrayList<>(0);
        Map<String, AppToken> tokens = redisTemplate.opsForHash().entries(TOKEN_HASH_KEY);
        for (Map.Entry<String, AppToken> kvs : tokens.entrySet()) {
            AppToken appToken = kvs.getValue();
            // 超过10分钟
            if (currentTime - appToken.getLastTime() > 600000L) {
                clearTokenKeys.add(kvs.getKey());
            }
        }
        for (String tokenKey : clearTokenKeys) {
            redisTemplate.opsForHash().delete(TOKEN_HASH_KEY, tokenKey);
        }
        LOG.trace("本次共清理超时Token:{}个", clearTokenKeys.size());
    }
}
